from CFDB_Classes import *
from CFDB_Fetcher import *
from CFDB_Database import CFDB_Database
import sys
import json
import datetime

year = 2024

class NCAAFootball():
	def __init__(self):
		self.database = CFDB_Database()
		self.fetcher = CFDB_Fetcher()

	def InitializeDatabases(self):
		self.database.InitializeDatabase()

	def LoadInitialData(self):
		self.LoadInitialGamesData()
		self.LoadInitialTeamsData()
		self.LoadInitialCalendar()
		self.LoadInitialRecords()
		self.LoadInitialConferences()
		self.LoadInitialVenues()
		self.LoadInitialRankings()
		self.LoadScoreboard()

	def LoadInitialTeamsData(self):
		print('Retrieving team data...', end='', flush=True)
		rawTeamsData = self.fetcher.FetchTeams()
		teamsData = json.loads(rawTeamsData)

		print('done.\nAdding teams to database...', end='', flush=True)
		for teamData in teamsData:
			team = CFDB_Team()
			team.Parse(teamData)
			self.database.AddNewTeam(team)			
		print('done.')

	def LoadInitialGamesData(self):
		print('Retrieving game for entire season...', end='', flush=True)
		rawGameData = self.fetcher.FetchGames(year)
		gamesData = json.loads(rawGameData)

		print('done.\nAdding games to database...', end='', flush=True)
		for gameData in gamesData:
			game = CFDB_Game()
			game.Parse(gameData)
			self.database.AddNewGame(game)
		print('done.')

	def LoadInitialCalendar(self):
		print('Retrieving calendar data...', end='', flush=True)
		rawCalendarData = self.fetcher.FetchCalendar(year)
		calendarData = json.loads(rawCalendarData)
		
		print('done.\nAdding calendar to database...', end='', flush=True)
		for weekData in calendarData:
			week = CFDB_Week()
			week.Parse(weekData)
			self.database.AddCalendarWeek(week)
		print('done.')

	def LoadInitialRecords(self):
		print('Retrieving records data...', end='', flush=True)
		rawRecordsData = self.fetcher.FetchRecords(year)
		recordsData = json.loads(rawRecordsData)

		print('done.\nAdding records to database...', end='', flush=True)
		for recordData in recordsData:
			record = CFDB_TeamRecord()
			record.Parse(recordData)
			self.database.AddRecord(record)
		print('done.')

	def LoadInitialConferences(self):
		print('Retrieving conference data...', end='', flush=True)
		rawConferencesData = self.fetcher.FetchConferences()
		conferencesData = json.loads(rawConferencesData)

		print('done.\nAdding conferences to database...', end='', flush=True)
		for conferenceData in conferencesData:
			conference = CFDB_Conference()
			conference.Parse(conferenceData)
			self.database.AddConference(conference)
		print('done.')

	def LoadInitialVenues(self):
		print('Retrieving location data...', end='', flush=True)
		rawVenuesData = self.fetcher.FetchVenues()
		venuesData = json.loads(rawVenuesData)

		print('done.\nAdding venues to database...', end='', flush=True)
		for venueData in venuesData:
			venue = CFDB_Location()
			venue.Parse(venueData)
			self.database.AddVenue(venue)
		print('done.')

	def LoadInitialRankings(self):
		print('Retrieving rankings data...', end='', flush=True)
		rawRankingsData = self.fetcher.FetchRankings(year)
		rankingsData = json.loads(rawRankingsData)

		print('done.\nAdding rankings to database...', end='', flush=True)
		for rankingData in rankingsData:
			rankingWeek = CFDB_RankingWeek()
			rankingWeek.Parse(rankingData)
			for poll in rankingWeek.polls:
				for rank in poll.ranks:
					self.database.AddRanking(
						rankingWeek.season, 
						rankingWeek.seasonType,
						rankingWeek.week,
						poll.poll,
						rank
						)
		print('done.')

	def LoadScoreboard(self):
		print('Retrieving scoreboard data...', end='', flush=True)
		rawScoreboardData = self.fetcher.FetchScoreboard()
		scoreboardData = json.loads(rawScoreboardData)
		self.database.ClearScoreboard()

		print('done.\nAdding scoreboard data to database...', end='', flush=True)
		for gameData in scoreboardData:
			game = CFDB_ScoreboardGame()
			game.Parse(gameData)
			self.database.UpdateScoreboardGame(game)
		print('done.')

	def UpdateGamesData(self):
		print('Retrieving updated games data...', end='', flush=True)
		rawGamesData = self.fetcher.FetchGames(year)
		gamesData = json.loads(rawGamesData)

		print('done.\nUpdating games in database...', end='', flush=True)
		for gameData in gamesData:
			game = CFDB_Game()
			game.Parse(gameData)
			self.database.UpdateGameData(game)
		print('done.')

	def UpdateRecordsData(self):
		print('Retrieving updated records data...', end='', flush=True)
		rawRecordsData = self.fetcher.FetchRecords(year)
		recordsData = json.loads(rawRecordsData)

		print('done.\nUpdating records in database...', end='', flush=True)
		for recordData in recordsData:
			record = CFDB_TeamRecord()
			record.Parse(recordData)
			self.database.UpdateRecordData(record)
		print('done.')

	def UpdateRankingsData(self):
		if self.database.ClearRankings() == 0:
			self.LoadInitialRankings()

	def CheckScoreboard(self):
		games = self.database.GetScoreboardGames()
		gamesOver = 0
		gamesStarted = 0
		gamesNotPlayed = 0

		if games == None or len(games) == 0:
			return 2
		totalGames = len( games )
		for game in games:
			if game.status == "completed":
				gamesOver += 1
			elif game.period != None and game.period != 0:
				gamesStarted += 1
			else:
				gamesNotPlayed += 1

		if totalGames == gamesOver:
			return 1
		if gamesStarted > 0:
			return 2
		return 0

if __name__ == "__main__":
	football = NCAAFootball()
	time = datetime.datetime.now()
	for arg in sys.argv:
		if arg == "-i" or arg == "i":
			football.InitializeDatabases()
			football.LoadInitialData()
		elif arg == "-u" or arg == "u":
			result = football.CheckScoreboard()
			print("Result: {0}".format(result))
			if result == 2:
				football.LoadScoreboard()
				football.UpdateGamesData()
				football.UpdateRecordsData()
			elif result == 1:
				if time.minute >= 5 and time.minute < 10:
					football.LoadScoreboard()
			else:
				if (time.minute % 15) >= 0 and (time.minute % 15) <5:
					football.LoadScoreboard()
			if time.weekday() == 6 and time.hour == 21:
				if time.minute >= 5 and time.minute < 10:
					football.UpdateRankingsData()
		elif arg == "-d" or arg == "d":
			football.UpdateRankingsData()
		elif arg == "-a" or arg == "a":
			football.LoadScoreboard()
			football.UpdateGamesData()
			football.UpdateRecordsData()
			
